#!/usr/bin/env python

import array
import random,math
import gtkhst

def test_hst(H=None):
    mean = 0.
    stdv = 2.
    print "\nTesting Histogram class: <x>=%.2f +/ %.2f" % (mean,stdv)

    if not H:
        H = gtkhst.Histogram(51,-25.5,25.5)

    i=0
    while i<50000:
        i+=1
        v = random.gauss(mean,stdv)
        H.fill(v,1.)

    print "N. entries: %f" % H.n()
    print "Mean      : %f" % H.mean()
    print "stdev     : %f" % H.stdev()
    print "Value range: [%f,%f]" % (H.xmin(),H.xmax())

def test_tracer():
    mean = 1.
    stdv = 1.
    print "\nTesting Tracer class: <x>=%.2f +/ %.2f" % (mean,stdv)

    H = gtkhst.Tracer(50)

    i=0
    while i<50000:
        i+=1
        v = random.gauss(mean,stdv)
        H.fill(v,1.)

    print "N. entries: %f" % H.n()
    print "Mean      : %f" % H.mean()
    print "stdev     : %f" % H.stdev()
    print "Value range: [%f,%f]" % (H.xmin(),H.xmax())


def test_h2d(h2d=None):
    mean1 = 1.
    sg1   = 2.
    mean2 = 2.
    sg2   = 1.
    corr  = 0.75
    f1 = math.sqrt( (1.+corr)/2. )
    f2 = math.sqrt( (1.-corr)/2. )
    print ""
    print "Stats <x>=%.2f +/- %.2f <y>=%.2f +/- %.2f corr=%.2f" % (
        mean1,sg1,mean2,sg2,corr)

    if not h2d:
        h2d = gtkhst.Histogram2D( 50, mean1-5.*sg1,mean1+5.*sg1,
                               50, mean2+5.*sg2,mean2+5.*sg2)
    
    S = gtkhst.Stats()
    i = 0
    while i<50000:
        i += 1
        x = random.gauss(0.,1.)
        y = random.gauss(0.,1.)
        h2d.fill(mean1 + sg1*(f1*x+f2*y), mean2 + sg2*( f1*x-f2*y),1.)

    h2d.get_stats(S)
    print ("N. entries   : %f" % S.get_nc())
    print ("X Mean       : %f" % S.get_mean(0))
    print ("X stdev      : %f" % S.get_stdev(0))
    print ("Y Mean       : %f" % S.get_mean(1))
    print ("Y stdev      : %f" % S.get_stdev(1))
    print ("XY corr.     : %f" % S.get_correlation())
    print ("X value range: [%f,%f]" % (S.get_min(0),S.get_max(0)))
    print ("Y value range: [%f,%f]" % (S.get_min(1),S.get_max(0)))
    

def f2d(x,y):
    return x+y
    
def main():

    print '\n\nTesting 1D functions: sin(PI/2)=',
    f = gtkhst.Function(20,0.,20.,math.sin)
    print f.eval(math.pi/2.)

    x=1.
    y=2.
    print ('\n\nTesting 2D functions: f(%f,%f)=x+y=' % (x,y)),
    f2 = gtkhst.Function2D(20,0.,10.,40,0.,20.,f2d)
    print f2.eval(x,y)

    test_hst()
    test_tracer()
    test_h2d()

    print '\n\nTesting buffers and 1d histograms'
    a = array.array('d',[0 for x in range(0,20)])
    b = buffer(a)

    print a

    print "We should get a complain here"
    try: 
        H = gtkhst.Histogram(40,0.,120.,a)
    except RuntimeError, s:
        print "Aparently the buffer is smaller than required"
        print "this is the message of the complain:"
        print s

    print "\nIt should work now"
    H = gtkhst.Histogram(20,-20.,20.,a)
    test_hst(H)
    print a

    print '\n\nTesting buffers and 2d histograms'
    a = array.array('d',[0 for x in range(0,25)])
    b = buffer(a)
    
    print "We should ge a complain here"
    try: 
        H = gtkhst.Histogram2D(40,0.,120.,40,0.,120,a)
    except RuntimeError, s:
        print "Aparently the buffer is smaller than required"
        print "this is the message of the complain:"
        print s

    print "\nIt should work now"
    H = gtkhst.Histogram2D(5,-20.,20.,5,-20,20,a)
    test_h2d(H)
    print a

if __name__=="__main__":
    main()

#---------------------------------------------------------------------------
