#!/usr/bin/env python

import os, gtk
import random
import math
import gobject
import gtkhst

class SFunc:
    def __init__(self,freq,mean,sigma):
        self.freq=freq
        self.mean=mean
        self.sigma=sigma
        self.ff = gtkhst.Histogram(51,-25.5,25.5)

def pulse(x):
    v1=0.
    td1=25.
    tau1=5.
    v2=10.
    td2=50.
    tau2=10.

    f = v1

    if x> td1:
        f += (v2-v1)*(1.-math.exp( -(x-td1)/tau1 ))

    if x>td2:
        f += (v1-v2)*(1.-math.exp( -(x-td2)/tau2))

    return f

class MainWindow(dict):
    def __init__(self, lf, glade_path="tsthst.glade", root="window1", domain=None):
        self.lf = lf
        self.tracer = gtkhst.Tracer(51)
        self.hst2d  = gtkhst.Histogram2D(51,-25.,25.,51,-25.,25.)
        
        self.create()

    def __getattr__(self, data_name):
        if data_name in self:
            data = self[data_name]
            return data
        else:
            widget = self.window.get_data(data_name)
            if widget != None:
                self[data_name] = widget
                return widget
            else:
                raise AttributeError, data_name

    def __setattr__(self, name, value):
        self[name] = value

    def create(self):
        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.set_title("tsthst");
        self.window.connect('delete_event',self.gtk_true)
        
        vbox = gtk.VBox(False,5)
        self.window.add(vbox)
        self.window.set_data('vbox',vbox)
        
        table = gtk.Table(4,3,False)
        vbox.pack_start(table,True,True,0)
        self.window.set_data('table',table)
        
        # Frame for the histogram
        frmHst = gtk.Frame()
        frmHst.set_shadow_type(gtk.SHADOW_IN)
        table.attach(frmHst,0,1,0,1,gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 0, 0)
        self.window.set_data('frmHst',frmHst)
        
        hst = self.create_histogram("hst")
        frmHst.add(hst)
        
        frmHst.set_label_widget(gtk.Label("A histogram"))
        
        # frame for the tracer
        frmTracer = gtk.Frame()
        frmTracer.set_shadow_type(gtk.SHADOW_IN)
        table.attach(frmTracer,1,2,0,1,gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 0, 0)
        self.window.set_data('frmTracer',frmTracer)
        
        tracer = self.create_tracer("tracer")
        frmTracer.add(tracer)
        frmTracer.set_label_widget(gtk.Label("A tracer"))
        
        # frame for the function
        frmFunc = gtk.Frame()
        frmFunc.set_shadow_type(gtk.SHADOW_IN)
        frmFunc.set_label_widget( gtk.Label("A function") )
        table.attach(frmFunc,2,3,0,1,gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 0, 0)
        self.window.set_data('frmFunc',frmFunc)
        
        func = self.create_func('func')
        frmFunc.add(func)
        
        # frame for the 2D histogram
        frmHst2D = gtk.Frame()
        frmHst2D.set_shadow_type(gtk.SHADOW_IN)
        frmHst2D.set_label_widget( gtk.Label("A 2D histogram") )
        table.attach(frmHst2D,0,1,1,2,gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 0, 0)
        self.window.set_data('frmHst2D',frmHst2D)
        
        hst2d = self.create_hst2d("hst2d")
        frmHst2D.add(hst2d)
        
        # frame for the graph
        t = [ 0.1*x for x in range(0,250) ]
        x = [ math.exp( 0.1*v ) * math.cos( v ) for v in t ]
        y = [ math.exp( 0.1*v ) * math.sin( v ) for v in t ]
        frmGraph = gtk.Frame()
        frmGraph.set_label_widget( gtk.Label("A Graph") )
        table.attach(frmGraph, 1, 2, 1, 2, gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 0, 0)
        self.window.set_data('frmGraph',frmGraph)
        
        graph = gtkhst.Hst1D( gtkhst.Graph( len(t), x, y) )
        frmGraph.add( graph )
        
        # frame for the gauge
        frmGauge = gtk.Frame()
        frmGauge.set_shadow_type(gtk.SHADOW_IN)
        frmGauge.set_label_widget( gtk.Label("A gauge") )
        frmGauge.add( self.create_gauge('gauge') )
        table.attach(frmGauge,2, 3, 1, 2, gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 0, 0)
        self.window.set_data('frmGauge',frmGauge)

        # the Profile
        frmProf = gtk.Frame()
        frmProf.set_shadow_type(gtk.SHADOW_IN)
        frmProf.set_label_widget( gtk.Label("A gauge") )
        frmProf.add( self.create_profile('profile') )
        table.attach( frmProf, 1, 2, 2, 3, gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 0, 0)
        self.window.set_data('frmProf', frmProf)

        table.attach( gtk.Label('X'), 0, 1, 2, 3, gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 0, 0)
        table.attach( gtk.Label('X'), 2, 3, 2, 3, gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 0, 0)

        
        # The doc window
        scroll = gtk.ScrolledWindow()
        scroll.set_policy(gtk.POLICY_NEVER,gtk.POLICY_NEVER)
        table.attach(scroll,0, 3, 3, 4,gtk.EXPAND|gtk.FILL, gtk.EXPAND|gtk.FILL, 0, 0)
        self.window.set_data('scroll',scroll)
        
        textview = gtk.TextView()
        textview.set_border_width(5)
        textview.set_editable(False)
        textview.set_wrap_mode(gtk.WRAP_WORD)
        textview.set_cursor_visible(False)
        textview.set_left_margin(5)
        textview.set_right_margin(5)
        textview.get_buffer().set_text("You can choose a given range in any of the axis by selecting it with the left button of the mouse while preessing\n\tCTRL  for the X axis or\n\tSHIFT for the Y axis\nThe original range is restored presing the mouse middle button.\nClick the mouse right button to explore the popup menu and change some of the histogram attributes.",-1)
        
        scroll.add(textview)
        
        btnbox = gtk.HButtonBox()
        vbox.pack_start(btnbox,False,True)
        
        btn = gtk.Button(stock=gtk.STOCK_OK)
        btn.connect('clicked',self.gtk_main_quit)
        
        btnbox.pack_start(btn)
        
        self.window.show_all()
        
        return

    def gtk_true(self, *args):
        return True

    def gtk_false(self, *args):
        return False

    def gtk_main_quit(self, *args):
        gtk.main_quit()

    def main(self):
        gtk.main()

    def quit(self):
        gtk.main_quit()

    def run(self):
        try:
            self.main()
        except KeyboardInterrupt:
            self.on_keyboard_interrupt()

    def on_keyboard_interrupt(self):
        pass


    def new(self):
        pass

    def create_histogram(self, str):
        widget = gtkhst.Hst1D(self.lf[0].ff)
        for x in self.lf[1:]:
            widget.add_curve(x.ff)

        widget.filled(True)
        widget.set_xtitle("X axis");
        widget.set_ytitle("Y axis");
        widget.monitor_start(100)
        for color in ("medium sea green", "royal blue","firebrick",
                      "gold", "turquoise"):
            widget.add_curve_color(color)
##         widget.set_curve_colors("medium sea green",
## 			       "royal blue",
## 			       "firebrick",
## 			       "gold",
## 			       "turquoise")
        self.window.set_data(str,widget)
        return widget
       

    def create_tracer(self, str):
        widget = gtkhst.Hst1D(self.tracer)
        widget.draw_y_axis(gtkhst.AXIS_MIDDLE)
        widget.set_xtitle("X axis");
        widget.set_ytitle("Y axis");
        widget.set_colors(foreground="red",background="black",axis="green");
        widget.monitor_start(150)
        self.window.set_data(str,widget)
        return widget

    def create_func(self, str):
        widget = gtkhst.Hst1D( gtkhst.Function( 51, 0., 100., pulse) )
        widget.set_xtitle("X axis");
        widget.set_ytitle("Y axis");
        self.window.set_data(str,widget)
        return widget

    def create_hst2d(self,str):
        widget = gtkhst.Hst2D( self.hst2d )
        widget.monitor_start(500)
        self.window.set_data(str,widget)
        return widget

    def create_gauge(self,str):
        widget = gtkhst.Gauge( 0.,100. )
        widget.set_title("gauge")
        self.window.set_data(str,widget)
        return widget

    def create_profile(self, str):
        self.hprof = gtkhst.Profile(50,0.,10.)
        widget = gtkhst.Hst1D( self.hprof )
        widget.monitor_start(500)
        self.window.set_data(str,widget)
        return widget

def gaus2d(m1,s1,m2,s2,corr):
    f1 = math.sqrt( (1.+corr)/2. )
    f2 = math.sqrt( (1.-corr)/2. )
    x = random.gauss(0.,1.)
    y = random.gauss(0.,1.)
    return ( m1 + s1*(f1*x+f2*y), m2 + s2*( f1*x-f2*y) )
        
p2 = 2.*math.pi
tim = 0.
dtim = 2.*math.pi/100.
def idle_func(w):
    global p2,tim,dtim
    
    r = random.random()
    for x in w.lf:
        if r<=x.freq:
            x.ff.fill(random.gauss(x.mean,x.sigma),1)
                        
    tim += dtim
    if tim>p2:
        tim = 0.

    w.tracer.fill(math.sin(tim),1.)

    x,y = gaus2d(-10,15.,-10,5.,-0.75)
    w.hst2d.fill(x,y,1.)
    x,y = gaus2d(10,15.,10,5.,0.75)
    w.hst2d.fill(x,y,1.)

    w.gauge.set_value(random.gauss(5.,1.))

    # the profile
    nt = w.hprof.npts()
    np = int( nt*random.random() )
    for i in range(0,np):
        ib = int( nt*random.random() )
        x  = w.hprof.bin_center(ib)
        w.hprof.fill(x, random.gauss(-(x-5.)*(x-5.)+10., 2.),1)
    
    return True



def main():
    lf = [
        SFunc(1.,0.,2.),
        SFunc(0.5,10.,2.),
        SFunc(0.75,-10.,2.)
        ]

    i = 0
    for func in lf:
        func.ff.set_name("Function %d" % i )
        i += 1
    window = MainWindow(lf)
    gobject.idle_add(idle_func,window)
    print 'starting'
    window.run()

if __name__ == "__main__":
    main()

#--------------------------------------------------------
